#ifndef MSP_CORE_ALGORITHM_H_
#define MSP_CORE_ALGORITHM_H_

#include <algorithm>

namespace Msp {

template<typename Container, typename T>
inline typename Container::iterator find(Container &cont, const T &value)
{
	return std::find(cont.begin(), cont.end(), value);
}

template<typename Container, typename T>
inline typename Container::const_iterator find(const Container &cont, const T &value)
{
	return std::find(cont.begin(), cont.end(), value);
}

template<typename Container, typename Predicate>
inline typename Container::iterator find_if(Container &cont, Predicate pred)
{
	return std::find_if(cont.begin(), cont.end(), pred);
}

template<typename Container, typename Predicate>
inline typename Container::const_iterator find_if(const Container &cont, Predicate pred)
{
	return std::find_if(cont.begin(), cont.end(), pred);
}

template<typename Container, typename T>
inline typename Container::iterator lower_bound(Container &cont, const T &value)
{
	return std::lower_bound(cont.begin(), cont.end(), value);
}

template<typename Container, typename T>
inline typename Container::const_iterator lower_bound(const Container &cont, const T &value)
{
	return std::lower_bound(cont.begin(), cont.end(), value);
}

template<typename Container, typename T, typename Predicate>
inline typename Container::iterator lower_bound(Container &cont, const T &value, Predicate pred)
{
	return std::lower_bound(cont.begin(), cont.end(), value, pred);
}

template<typename Container, typename T, typename Predicate>
inline typename Container::const_iterator lower_bound(const Container &cont, const T &value, Predicate pred)
{
	return std::lower_bound(cont.begin(), cont.end(), value, pred);
}

template<typename Container, typename T>
inline typename Container::iterator upper_bound(Container &cont, const T &value)
{
	return std::upper_bound(cont.begin(), cont.end(), value);
}

template<typename Container, typename T>
inline typename Container::const_iterator upper_bound(const Container &cont, const T &value)
{
	return std::upper_bound(cont.begin(), cont.end(), value);
}

template<typename Container, typename T, typename Predicate>
inline typename Container::iterator upper_bound(Container &cont, const T &value, Predicate pred)
{
	return std::upper_bound(cont.begin(), cont.end(), value, pred);
}

template<typename Container, typename T, typename Predicate>
inline typename Container::const_iterator upper_bound(const Container &cont, const T &value, Predicate pred)
{
	return std::upper_bound(cont.begin(), cont.end(), value, pred);
}

template<typename Container>
inline void sort(Container &cont)
{
	std::sort(cont.begin(), cont.end());
}

template<typename Container, typename Predicate>
inline void sort(Container &cont, Predicate pred)
{
	std::sort(cont.begin(), cont.end(), pred);
}

template<typename Container>
inline void stable_sort(Container &cont)
{
	std::stable_sort(cont.begin(), cont.end());
}

template<typename Container, typename Predicate>
inline void stable_sort(Container &cont, Predicate pred)
{
	std::stable_sort(cont.begin(), cont.end(), pred);
}

} // namespace Msp

#endif
